function calc_discharge_timeseries(migrated_pixels)

if ispc
    addpath(genpath('D:/work/Fellowship_Leeds/Data'));
else
    addpath(genpath('/resstore/b0133/Data/MATLAB_tools'));
end

%% Set physical parameters

% Ice density
rhoi = 917; % kg/m^3

%% Load flux gate data

% Gate locations
load('../flux_gates/flux_gates_migrated.mat');
gate_x = gate_x_migrate{migrated_pixels};
gate_y = gate_y_migrate{migrated_pixels};
[gate_lat,gate_lon] = ps2ll(gate_x,gate_y);
gate_nan = isnan(gate_x); % indices of gate nans
clear gate_x_migrate gate_y_migrate
migration_distance = 0:200:3000; % metres
migration_distance = migration_distance(migrated_pixels);

% Reference data (various versions of bed topograpghy, one surface, gate angle)
ref = load(['../reference_data/reference_gate_data_migrate_' num2str(migration_distance) 'm.mat']);

% ice velocity timeseries
V = load(['../velocity/gate_vel_filled_migrate_' num2str(migration_distance) 'm_SID.mat']);

% gate-to-grounding line thickness change due to SMB
load(['../reference_data/downstream_smb_correction/downstream_smb_correction_' num2str(migration_distance) 'm.mat']); % gate_smb

% dHdt at gate
dHdt = load('../reference_data/dHdt/gate_dHdt_ts.mat');
dHdt.gate_dHdt_ts = dHdt.gate_dHdt_ts{migrated_pixels};
dHdt.gate_dHdtE_ts = dHdt.gate_dHdtE_ts{migrated_pixels};

% firn air content
FAC_FDM = load(['../FAC/FAC_migrate_' num2str(migration_distance) 'm.mat']); % FAC from IMAU FDM
FAC_GSFC = load(['../FAC/FAC_GSFC_migrate_' num2str(migration_distance) 'm.mat']); % FAC from GSFC with MERRA-2

% Calculate discharge at each velocity epoch, so set time here
time = V.time_vel;


%% Calculate cross-gate velocity
% The aim is to provide an estimate of the total discharge across the
% grounding line, so always use the 'filled' velocity. Later, we'll
% calculate how much of the discharge has been observed vs how much is
% 'filled'

gate_theta = repmat(ref.gate_theta,[1,length(time)]); % angle of flux gate; gate never changes, so keep constant through time and assume no error in this angle
V.ctv = abs(V.vx_filled.*sin(gate_theta) - V.vy_filled.*cos(gate_theta));
V.ctvE = abs(V.vxE_filled.*sin(gate_theta) - V.vyE_filled.*cos(gate_theta));


%% Calculate time varying ice thickness

% Get initial thickness for each bed product
% These are based on the REMA surface, which is approximately timestamped
% to May 9th 2015 (https://tc.copernicus.org/articles/13/665/2019/)
H_raw = repmat(ref.gate_H_raw(:),[1,length(time)]);
H_adj = repmat(ref.gate_H_adj(:),[1,length(time)]);
H_BM  = repmat(ref.gate_H_BM(:), [1,length(time)]);
H_B2  = repmat(ref.gate_H_B2(:), [1,length(time)]);
H_raw_err = repmat(ref.gate_H_raw_err(:),[1,length(time)]);
H_adj_err = repmat(ref.gate_H_adj_err(:),[1,length(time)]);
H_BM_err  = repmat(ref.gate_H_BM_err(:), [1,length(time)]);
H_B2_err  = repmat(ref.gate_H_B2_err(:), [1,length(time)]);

% Calculate change in thickness relative to reference for each
% velocity/discharge epoch. 194877 is a useful test index for PIG
ref_time = datenum(2015,5,9);
dH = zeros(size(H_raw)).*NaN;
for i = 1:length(gate_x)
    if isfinite(gate_x(i))
        dHdt_ts = dHdt.gate_dHdt_ts(i,:);
        %dHdt_ts(dHdt.time<=datenum(1992,1,1))=NaN; % generally unrealistically fast rates of thinning before this because of very few data points
        dHdt_ts = interp1gap(dHdt.time,dHdt_ts,time);
        dHdt_ts = fillmissing(dHdt_ts,'Previous');
        dHdt_ts = fillmissing(dHdt_ts,'Next');
        dHdt_ts(time>max(dHdt.time)) = nanmean(dHdt_ts(time>=datenum(2015,1,1) & time<=max(dHdt.time))); % dHdt timeseries stops in 2020, so use the average dHdt from 2018 onwards for any time after 2020. i.e. assume thinning rate is maintained.
        dHdt_ts(time<min(dHdt.time)) = nanmean(dHdt_ts(time>=min(dHdt.time) & time<=datenum(1995,1,1))); % dHdt timeseries begins in 1985, so use the average dHdt from 1985 to 1988 for any time prior to 1985
        [~,ind] = findNearest(time,ref_time);
        dH(i,:) = cumsum(dHdt_ts.*(gradient(time)./365.25)); 
        dH(i,:) = dH(i,:) - dH(i,ind);
        %dH(i,time>max(dHdt.time)) = nanmean(dH(time>=datenum(2014,12,16) & time<=max(dHdt.time))); % dHdt timeseries stops in 2020, so use the average dHdt from 2015 onwards for any time after 2020; this overrides the equivalent lines above. i.e. assume no further thickness changes occur
        %dH(i,time<min(dHdt.time)) = nanmean(dH(time>=min(dHdt.time) & time<=datenum(1991,3,17))); % dHdt timeseries begins in 1985, so use the average dHdt from 1992 to 1995 for any time prior to 1985; this overrides the equivalent lines above
    end
end

% Apply thickness change
H_raw = H_raw + dH;
H_adj = H_adj + dH;
H_BM = H_BM + dH;
H_B2 = H_B2 + dH;
%   - other versions for calculating contribution of corrections
H_adjS = H_adj; % 'static'; adjusted thickness with no corrections applied
H_adjIS = H_adj + dH; % 'in situ'; same as adj, but with no downstream SMB applied
H_adjNDH = H_adj; % 'No dH'; same as adj, but with no dH
H_adjNF = H_adj + dH; % 'no firn'; same as adj, but with no firn correction

% Adjust thickness error - assume 0.1 m error for each year of elevation change
time_diff    = abs((time - ref_time)./365.25);
time_diff    = repmat(time_diff,[length(gate_x),1]);
H_adjS_err   = H_adj_err;
H_adjNDH_err = H_adj_err;
H_adjNF_err  = sqrt(H_adj_err.^2 + (time_diff.*0.1).^2);
H_raw_err    = sqrt(H_raw_err.^2 + (time_diff.*0.1).^2);
H_adj_err    = sqrt(H_adj_err.^2 + (time_diff.*0.1).^2);
H_BM_err     = sqrt(H_BM_err.^2  + (time_diff.*0.1).^2);
H_B2_err     = sqrt(H_B2_err.^2  + (time_diff.*0.1).^2);
H_adjIS_err  = H_adj_err; % include dH error
H_adj_GSFC_err  = H_adj_err; % initialise error for discharge using GSFC firn model; include dH error

% Apply firn correction
%   - the baseline thickness estimates above already have the
%   climatological firn content removed, so we need to add that back on
%   first
H_raw = H_raw + repmat(FAC_FDM.fac_clim,[1,length(time)]);
H_adj = H_adj + repmat(FAC_FDM.fac_clim,[1,length(time)]);
H_BM  = H_BM  + repmat(FAC_FDM.fac_clim,[1,length(time)]);
H_B2  = H_B2  + repmat(FAC_FDM.fac_clim,[1,length(time)]);
H_adjS   = H_adjS   + repmat(FAC_FDM.fac_clim,[1,length(time)]);
H_adjIS  = H_adjIS  + repmat(FAC_FDM.fac_clim,[1,length(time)]);
H_adjNDH = H_adjNDH + repmat(FAC_FDM.fac_clim,[1,length(time)]);
H_adjNF  = H_adjNF  + repmat(FAC_FDM.fac_clim,[1,length(time)]);
H_adj_GSFC = H_adj; % initialise version for applying the GSFC firn model
%   - then apply the time-varying firn correction
for i = 1:length(gate_x)
    if isfinite(gate_x(i))
        % load FAC and interpolate to velocity/discharge timeseries
        FAC_ts = FAC_FDM.fac_monthly(i,:);
        FAC_ts(FAC_ts==0)=NaN;
        FAC_ts = interp1gap(FAC_FDM.fac_time,FAC_ts,time);
        % As the velocity time-series extends beyond the FAC timeseries,
        % the more recent FAC time points will be NaN. Fill those with the
        % monthly FAC climatology.
        isn = isnan(FAC_ts);
        if sum(isn)>0
            [~,M,~] = datevec(time(isn));
            FAC_ts(find(isn==1)) = FAC_FDM.fac_monthly_clim(i,M);
        end
        % Apply firn correction
        H_raw(i,:) = H_raw(i,:) - FAC_ts;
        H_adj(i,:) = H_adj(i,:) - FAC_ts;
        H_BM(i,:)  = H_BM(i,:)  - FAC_ts;
        H_B2(i,:)  = H_B2(i,:)  - FAC_ts;
        H_adjIS(i,:)  = H_adjIS(i,:)  - FAC_ts;
        H_adjNDH(i,:) = H_adjNDH(i,:) - FAC_ts;
        % Repeat for GSFC firn model, but only for adjusted thickness data
        FAC_ts = FAC_GSFC.fac_monthly(i,:);
        FAC_ts(FAC_ts==0)=NaN;
        FAC_ts = interp1gap(FAC_GSFC.fac_time,FAC_ts,time);
        isn = isnan(FAC_ts);
        if sum(isn)>0
            [~,M,~] = datevec(time(isn));
            FAC_ts(find(isn==1)) = FAC_GSFC.fac_monthly_clim(i,M);
        end
        H_adj_GSFC(i,:)    = H_adj_GSFC(i,:)    - FAC_ts;
    end
end

% Account for thickness changes between the gate and the grounding line
H_raw = H_raw + repmat(gate_smb,[1,length(time)]);
H_adj = H_adj + repmat(gate_smb,[1,length(time)]);
H_BM  = H_BM  + repmat(gate_smb,[1,length(time)]);
H_B2  = H_B2  + repmat(gate_smb,[1,length(time)]);
H_adjNF  = H_adjNF  + repmat(gate_smb,[1,length(time)]);
H_adjNDH = H_adjNDH + repmat(gate_smb,[1,length(time)]);
H_adj_GSFC = H_adj_GSFC + repmat(gate_smb,[1,length(time)]);
% The code below was useful when using a time-varying correction for this
% mass flux. However, it was still rather simplified and the difference was
% so small that it didn't warrant the additional complexity
% for i = 1:length(gate_x)
%     if isfinite(gate_x(i))
%         gate_dH_ts = gate_correction.gate_dH_ts(i,:);
%         gate_dH_ts = interp1gap(gate_correction.time,gate_dH_ts,time);
%         gate_dH_ts = fillmissing(gate_dH_ts,'Previous');
%         gate_dH_ts = fillmissing(gate_dH_ts,'Next');
%         H_rawS(i,:) = H_rawS(i,:) + gate_dH_ts;
%         H_rawFO(i,:) = H_rawFO(i,:) + gate_dH_ts;
%         H_rawDHO(i,:) = H_rawDHO(i,:) + gate_dH_ts;
%         H_raw(i,:) = H_raw(i,:) + gate_dH_ts;
%         H_adj(i,:) = H_adj(i,:) + gate_dH_ts;
%         H_BM(i,:)  = H_BM(i,:)  + gate_dH_ts;
%         H_B2(i,:)  = H_B2(i,:)  + gate_dH_ts;
%     end
% end

% After all those corrections, check that we haven't created any
% unrealistic ice thickness
H_raw(H_raw<0) = 0;
H_adj(H_adj<0) = 0;
H_BM(H_BM<0)   = 0;
H_B2(H_B2<0)   = 0;
H_adjS(H_adjS<0)     = 0;
H_adjIS(H_adjIS<0)   = 0;
H_adjNDH(H_adjNDH<0) = 0;
H_adjNF(H_adjNF<0)   = 0;
H_adj_GSFC(H_adj_GSFC<0)   = 0;

% Finally, make sure that we have NaNs in the right place
for i = 1:length(time)
    H_raw(gate_nan,i) = NaN;
    H_adj(gate_nan,i) = NaN;
    H_BM(gate_nan,i)  = NaN;
    H_B2(gate_nan,i)  = NaN;
    H_adjS(gate_nan,i)   = NaN;
    H_adjIS(gate_nan,i)  = NaN;
    H_adjNDH(gate_nan,i) = NaN;
    H_adjNF(gate_nan,i)  = NaN;
    H_adj_GSFC(gate_nan,i)  = NaN;
end

%% Calculate discharge

% Get flux gate sections
sections = regionprops(~isnan(gate_x), gate_x,'pixelIdxList','PixelValues');

for i = 1:numel(sections)
    
    % Get section data
    sect_lat = gate_lat(sections(i).PixelIdxList);
    sect_lon = gate_lon(sections(i).PixelIdxList);
    %   - merged bed, REMA surface, without adjustments
    sect_H_raw = H_raw(sections(i).PixelIdxList,:);
    sect_H_raw_err = H_raw_err(sections(i).PixelIdxList,:);
    sect_H_raw_low = sect_H_raw-sect_H_raw_err;
    sect_H_raw_low(sect_H_raw_low<0) = 0; % prepare this now so that we don't end up with any negative discharge
    %   - merged bed, REMA surface, with adjustments
    sect_H_adj = H_adj(sections(i).PixelIdxList,:);
    sect_H_adj_err = H_adj_err(sections(i).PixelIdxList,:);
    sect_H_adj_low = sect_H_adj-sect_H_adj_err;
    sect_H_adj_low(sect_H_adj_low<0) = 0; % prepare this now so that we don't end up with any negative discharge
    %   - bedmachine thickness
    sect_H_BM = H_BM(sections(i).PixelIdxList,:);
    sect_H_BM_err = H_BM_err(sections(i).PixelIdxList,:);
    sect_H_BM_low = sect_H_BM-sect_H_BM_err;
    sect_H_BM_low(sect_H_BM_low<0) = 0; % prepare this now so that we don't end up with any negative discharge
    %   - bedmap2 thickness
    sect_H_B2 = H_B2(sections(i).PixelIdxList,:);
    sect_H_B2_err = H_B2_err(sections(i).PixelIdxList,:);
    sect_H_B2_low = sect_H_B2-sect_H_B2_err;
    sect_H_B2_low(sect_H_B2_low<0) = 0; % prepare this now so that we don't end up with any negative discharge
    %   - no thickness corrections applied at all
    sect_H_adjS = H_adjS(sections(i).PixelIdxList,:);
    sect_H_adjS_err = H_adjS_err(sections(i).PixelIdxList,:);
    sect_H_adjS_low = sect_H_adjS-sect_H_adjS_err;
    sect_H_adjS_low(sect_H_adjS_low<0) = 0; % prepare this now so that we don't end up with any negative discharge
    %   - no downstream thickness correction applied
    sect_H_adjIS = H_adjIS(sections(i).PixelIdxList,:);
    sect_H_adjIS_err = H_adjIS_err(sections(i).PixelIdxList,:);
    sect_H_adjIS_low = sect_H_adjIS-sect_H_adjIS_err;
    sect_H_adjIS_low(sect_H_adjIS_low<0) = 0; % prepare this now so that we don't end up with any negative discharge
    %   - no dH applied
    sect_H_adjNDH = H_adjNDH(sections(i).PixelIdxList,:);
    sect_H_adjNDH_err = H_adjNDH_err(sections(i).PixelIdxList,:);
    sect_H_adjNDH_low = sect_H_adjNDH-sect_H_adjNDH_err;
    sect_H_adjNDH_low(sect_H_adjNDH_low<0) = 0; % prepare this now so that we don't end up with any negative discharge
    %   - No firn correction applied
    sect_H_adjNF = H_adjNF(sections(i).PixelIdxList,:);
    sect_H_adjNF_err = H_adjNF_err(sections(i).PixelIdxList,:);
    sect_H_adjNF_low = sect_H_adjNF-sect_H_adjNF_err;
    sect_H_adjNF_low(sect_H_adjNF_low<0) = 0; % prepare this now so that we don't end up with any negative discharge
     %   - GSFC firn correction applied
    sect_H_adjGSFC = H_adj_GSFC(sections(i).PixelIdxList,:);
    sect_H_adjGSFC_err = H_adj_GSFC_err(sections(i).PixelIdxList,:);
    sect_H_adjGSFC_low = sect_H_adjGSFC-sect_H_adjGSFC_err;
    sect_H_adjGSFC_low(sect_H_adjGSFC_low<0) = 0; % prepare this now so that we don't end up with any negative discharge
    

    % Ice velocity
    sect_ctv = V.ctv(sections(i).PixelIdxList,:);
    sect_ctv_err = V.ctvE(sections(i).PixelIdxList,:);
    sect_ctv_low = sect_ctv - sect_ctv_err;
    sect_ctv_low(sect_ctv_low<0) = 0; % prepare this now so that we don't end up with any negative discharge
    
    %  Calculate distance between flux gate points
    d = pathdistps(sect_lat(:),sect_lon(:)); % Distance along flux gate
    dx = gradient(d); % Distance between each flux gate data point
    dx = repmat(dx(:),[1,length(time)]);
    
    % Calculate discharge
    %   - merged bed, REMA surface, without adjustments
    tmp_raw = sect_ctv.*sect_H_raw.*dx.*rhoi.*1e-12;
    tmp_raw_vel_max = (sect_ctv+sect_ctv_err).*(sect_H_raw).*dx.*rhoi.*1e-12;
    tmp_raw_vel_min = sect_ctv_low.*(sect_H_raw).*dx.*rhoi.*1e-12;
    tmp_raw_H_max = sect_ctv.*(sect_H_raw+sect_H_raw_err).*dx.*rhoi.*1e-12;
    tmp_raw_H_min = sect_ctv.*sect_H_raw_low.*dx.*rhoi.*1e-12;
    %   - merged bed, REMA surface, with adjustments
    tmp_adj = sect_ctv.*sect_H_adj.*dx.*rhoi.*1e-12;
    tmp_adj_vel_max = (sect_ctv+sect_ctv_err).*(sect_H_adj).*dx.*rhoi.*1e-12;
    tmp_adj_vel_min = sect_ctv_low.*(sect_H_adj).*dx.*rhoi.*1e-12;
    tmp_adj_H_max = sect_ctv.*(sect_H_adj+sect_H_adj_err).*dx.*rhoi.*1e-12;
    tmp_adj_H_min = sect_ctv.*sect_H_adj_low.*dx.*rhoi.*1e-12;
    %   - bedmachine thickness
    tmp_BM = sect_ctv.*sect_H_BM.*dx.*rhoi.*1e-12;
    tmp_BM_vel_max = (sect_ctv+sect_ctv_err).*(sect_H_BM).*dx.*rhoi.*1e-12;
    tmp_BM_vel_min = sect_ctv_low.*(sect_H_BM).*dx.*rhoi.*1e-12;
    tmp_BM_H_max = sect_ctv.*(sect_H_BM+sect_H_BM_err).*dx.*rhoi.*1e-12;
    tmp_BM_H_min = sect_ctv.*sect_H_BM_low.*dx.*rhoi.*1e-12;
    %   - bedmap2 thickness
    tmp_B2 = sect_ctv.*sect_H_B2.*dx.*rhoi.*1e-12;
    tmp_B2_vel_max = (sect_ctv+sect_ctv_err).*(sect_H_B2).*dx.*rhoi.*1e-12;
    tmp_B2_vel_min = sect_ctv_low.*(sect_H_B2).*dx.*rhoi.*1e-12;
    tmp_B2_H_max = sect_ctv.*(sect_H_B2+sect_H_B2_err).*dx.*rhoi.*1e-12;
    tmp_B2_H_min = sect_ctv.*sect_H_B2_low.*dx.*rhoi.*1e-12;
    %   - no thickness corrections applied at all
    tmp_adjS = sect_ctv.*sect_H_adjS.*dx.*rhoi.*1e-12;
    tmp_adjS_vel_max = (sect_ctv+sect_ctv_err).*(sect_H_adjS).*dx.*rhoi.*1e-12;
    tmp_adjS_vel_min = sect_ctv_low.*(sect_H_adjS).*dx.*rhoi.*1e-12;
    tmp_adjS_H_max = sect_ctv.*(sect_H_adjS+sect_H_adjS_err).*dx.*rhoi.*1e-12;
    tmp_adjS_H_min = sect_ctv.*sect_H_adjS_low.*dx.*rhoi.*1e-12;
    %   - no downstream correction applied
    tmp_adjIS = sect_ctv.*sect_H_adjIS.*dx.*rhoi.*1e-12;
    tmp_adjIS_vel_max = (sect_ctv+sect_ctv_err).*(sect_H_adjIS).*dx.*rhoi.*1e-12;
    tmp_adjIS_vel_min = sect_ctv_low.*(sect_H_adjIS).*dx.*rhoi.*1e-12;
    tmp_adjIS_H_max = sect_ctv.*(sect_H_adjIS+sect_H_adjIS_err).*dx.*rhoi.*1e-12;
    tmp_adjIS_H_min = sect_ctv.*sect_H_adjIS_low.*dx.*rhoi.*1e-12;
    %   - no dH applied 
    tmp_adjNDH = sect_ctv.*sect_H_adjNDH.*dx.*rhoi.*1e-12;
    tmp_adjNDH_vel_max = (sect_ctv+sect_ctv_err).*(sect_H_adjNDH).*dx.*rhoi.*1e-12;
    tmp_adjNDH_vel_min = sect_ctv_low.*(sect_H_adjNDH).*dx.*rhoi.*1e-12;
    tmp_adjNDH_H_max = sect_ctv.*(sect_H_adjNDH+sect_H_adjNDH_err).*dx.*rhoi.*1e-12;
    tmp_adjNDH_H_min = sect_ctv.*sect_H_adjNDH_low.*dx.*rhoi.*1e-12;
    %   - no firn correction applied
    tmp_adjNF = sect_ctv.*sect_H_adjNF.*dx.*rhoi.*1e-12;
    tmp_adjNF_vel_max = (sect_ctv+sect_ctv_err).*(sect_H_adjNF).*dx.*rhoi.*1e-12;
    tmp_adjNF_vel_min = sect_ctv_low.*(sect_H_adjNF).*dx.*rhoi.*1e-12;
    tmp_adjNF_H_max = sect_ctv.*(sect_H_adjNF+sect_H_adjNF_err).*dx.*rhoi.*1e-12;
    tmp_adjNF_H_min = sect_ctv.*sect_H_adjNF_low.*dx.*rhoi.*1e-12;
    %   - GSFC firn correction applied
    tmp_adjGSFC = sect_ctv.*sect_H_adjGSFC.*dx.*rhoi.*1e-12;
    tmp_adjGSFC_vel_max = (sect_ctv+sect_ctv_err).*(sect_H_adjGSFC).*dx.*rhoi.*1e-12;
    tmp_adjGSFC_vel_min = sect_ctv_low.*(sect_H_adjGSFC).*dx.*rhoi.*1e-12;
    tmp_adjGSFC_H_max = sect_ctv.*(sect_H_adjGSFC+sect_H_adjGSFC_err).*dx.*rhoi.*1e-12;
    tmp_adjGSFC_H_min = sect_ctv.*sect_H_adjGSFC_low.*dx.*rhoi.*1e-12;
    
    % Concatenate
    if i == 1
        % merged bed, REMA surface, without adjustments
        Q_raw         = [tmp_raw;         nan(1,length(time))];
        Q_raw_vel_max = [tmp_raw_vel_max; nan(1,length(time))];
        Q_raw_vel_min = [tmp_raw_vel_min; nan(1,length(time))];
        Q_raw_H_max   = [tmp_raw_H_max;   nan(1,length(time))];
        Q_raw_H_min   = [tmp_raw_H_min;   nan(1,length(time))];
        % merged bed, REMA surface, with adjustments
        Q_adj         = [tmp_adj;         nan(1,length(time))];
        Q_adj_vel_max = [tmp_adj_vel_max; nan(1,length(time))];
        Q_adj_vel_min = [tmp_adj_vel_min; nan(1,length(time))];
        Q_adj_H_max   = [tmp_adj_H_max;   nan(1,length(time))];
        Q_adj_H_min   = [tmp_adj_H_min;   nan(1,length(time))];
        % bedmachine thickness
        Q_BM         = [tmp_BM;         nan(1,length(time))];
        Q_BM_vel_max = [tmp_BM_vel_max; nan(1,length(time))];
        Q_BM_vel_min = [tmp_BM_vel_min; nan(1,length(time))];
        Q_BM_H_max   = [tmp_BM_H_max;   nan(1,length(time))];
        Q_BM_H_min   = [tmp_BM_H_min;   nan(1,length(time))];
        % bedmap2 thickness
        Q_B2         = [tmp_B2;         nan(1,length(time))];
        Q_B2_vel_max = [tmp_B2_vel_max; nan(1,length(time))];
        Q_B2_vel_min = [tmp_B2_vel_min; nan(1,length(time))];
        Q_B2_H_max   = [tmp_B2_H_max;   nan(1,length(time))];
        Q_B2_H_min   = [tmp_B2_H_min;   nan(1,length(time))];
        % no thickness corrections
        Q_adjS         = [tmp_adjS;         nan(1,length(time))];
        Q_adjS_vel_max = [tmp_adjS_vel_max; nan(1,length(time))];
        Q_adjS_vel_min = [tmp_adjS_vel_min; nan(1,length(time))];
        Q_adjS_H_max   = [tmp_adjS_H_max;   nan(1,length(time))];
        Q_adjS_H_min   = [tmp_adjS_H_min;   nan(1,length(time))];
        % no downstream correction
        Q_adjIS         = [tmp_adjIS;         nan(1,length(time))];
        Q_adjIS_vel_max = [tmp_adjIS_vel_max; nan(1,length(time))];
        Q_adjIS_vel_min = [tmp_adjIS_vel_min; nan(1,length(time))];
        Q_adjIS_H_max   = [tmp_adjIS_H_max;   nan(1,length(time))];
        Q_adjIS_H_min   = [tmp_adjIS_H_min;   nan(1,length(time))];
        % no dH applied
        Q_adjNDH         = [tmp_adjNDH;         nan(1,length(time))];
        Q_adjNDH_vel_max = [tmp_adjNDH_vel_max; nan(1,length(time))];
        Q_adjNDH_vel_min = [tmp_adjNDH_vel_min; nan(1,length(time))];
        Q_adjNDH_H_max   = [tmp_adjNDH_H_max;   nan(1,length(time))];
        Q_adjNDH_H_min   = [tmp_adjNDH_H_min;   nan(1,length(time))];
        % no firn correction
        Q_adjNF         = [tmp_adjNF;         nan(1,length(time))];
        Q_adjNF_vel_max = [tmp_adjNF_vel_max; nan(1,length(time))];
        Q_adjNF_vel_min = [tmp_adjNF_vel_min; nan(1,length(time))];
        Q_adjNF_H_max   = [tmp_adjNF_H_max;   nan(1,length(time))];
        Q_adjNF_H_min   = [tmp_adjNF_H_min;   nan(1,length(time))];
        % GSFC firn correction
        Q_adjGSFC         = [tmp_adjGSFC;         nan(1,length(time))];
        Q_adjGSFC_vel_max = [tmp_adjGSFC_vel_max; nan(1,length(time))];
        Q_adjGSFC_vel_min = [tmp_adjGSFC_vel_min; nan(1,length(time))];
        Q_adjGSFC_H_max   = [tmp_adjGSFC_H_max;   nan(1,length(time))];
        Q_adjGSFC_H_min   = [tmp_adjGSFC_H_min;   nan(1,length(time))];
    else
        % merged bed, REMA surface without adjustments
        Q_raw         = [Q_raw;tmp_raw;                 nan(1,length(time))];
        Q_raw_vel_max = [Q_raw_vel_max;tmp_raw_vel_max; nan(1,length(time))];
        Q_raw_vel_min = [Q_raw_vel_min;tmp_raw_vel_min; nan(1,length(time))];
        Q_raw_H_max   = [Q_raw_H_max;tmp_raw_H_max;     nan(1,length(time))];
        Q_raw_H_min   = [Q_raw_H_min;tmp_raw_H_min;     nan(1,length(time))];
        % merged bed, REMA surface, with adjustments
        Q_adj         = [Q_adj;tmp_adj;                 nan(1,length(time))];
        Q_adj_vel_max = [Q_adj_vel_max;tmp_adj_vel_max; nan(1,length(time))];
        Q_adj_vel_min = [Q_adj_vel_min;tmp_adj_vel_min; nan(1,length(time))];
        Q_adj_H_max   = [Q_adj_H_max;tmp_adj_H_max;     nan(1,length(time))];
        Q_adj_H_min   = [Q_adj_H_min;tmp_adj_H_min;     nan(1,length(time))];
        % bedmachine thickness
        Q_BM         = [Q_BM;tmp_BM;                 nan(1,length(time))];
        Q_BM_vel_max = [Q_BM_vel_max;tmp_BM_vel_max; nan(1,length(time))];
        Q_BM_vel_min = [Q_BM_vel_min;tmp_BM_vel_min; nan(1,length(time))];
        Q_BM_H_max   = [Q_BM_H_max;tmp_BM_H_max;     nan(1,length(time))];
        Q_BM_H_min   = [Q_BM_H_min;tmp_BM_H_min;     nan(1,length(time))];
        % bedmap2 thickness
        Q_B2         = [Q_B2;tmp_B2;                 nan(1,length(time))];
        Q_B2_vel_max = [Q_B2_vel_max;tmp_B2_vel_max; nan(1,length(time))];
        Q_B2_vel_min = [Q_B2_vel_min;tmp_B2_vel_min; nan(1,length(time))];
        Q_B2_H_max   = [Q_B2_H_max;tmp_B2_H_max;     nan(1,length(time))];
        Q_B2_H_min   = [Q_B2_H_min;tmp_B2_H_min;     nan(1,length(time))];
        % no thickness corrections applied
        Q_adjS         = [Q_adjS;tmp_adjS;                 nan(1,length(time))];
        Q_adjS_vel_max = [Q_adjS_vel_max;tmp_adjS_vel_max; nan(1,length(time))];
        Q_adjS_vel_min = [Q_adjS_vel_min;tmp_adjS_vel_min; nan(1,length(time))];
        Q_adjS_H_max   = [Q_adjS_H_max;tmp_adjS_H_max;     nan(1,length(time))];
        Q_adjS_H_min   = [Q_adjS_H_min;tmp_adjS_H_min;     nan(1,length(time))];
        % no downstream correction applied
        Q_adjIS         = [Q_adjIS;tmp_adjIS;                 nan(1,length(time))];
        Q_adjIS_vel_max = [Q_adjIS_vel_max;tmp_adjIS_vel_max; nan(1,length(time))];
        Q_adjIS_vel_min = [Q_adjIS_vel_min;tmp_adjIS_vel_min; nan(1,length(time))];
        Q_adjIS_H_max   = [Q_adjIS_H_max;tmp_adjIS_H_max;     nan(1,length(time))];
        Q_adjIS_H_min   = [Q_adjIS_H_min;tmp_adjIS_H_min;     nan(1,length(time))];
        % no dH correction applied
        Q_adjNDH         = [Q_adjNDH;tmp_adjNDH;                 nan(1,length(time))];
        Q_adjNDH_vel_max = [Q_adjNDH_vel_max;tmp_adjNDH_vel_max; nan(1,length(time))];
        Q_adjNDH_vel_min = [Q_adjNDH_vel_min;tmp_adjNDH_vel_min; nan(1,length(time))];
        Q_adjNDH_H_max   = [Q_adjNDH_H_max;tmp_adjNDH_H_max;     nan(1,length(time))];
        Q_adjNDH_H_min   = [Q_adjNDH_H_min;tmp_adjNDH_H_min;     nan(1,length(time))];
        % no firn correction applied
        Q_adjNF         = [Q_adjNF;tmp_adjNF;                 nan(1,length(time))];
        Q_adjNF_vel_max = [Q_adjNF_vel_max;tmp_adjNF_vel_max; nan(1,length(time))];
        Q_adjNF_vel_min = [Q_adjNF_vel_min;tmp_adjNF_vel_min; nan(1,length(time))];
        Q_adjNF_H_max   = [Q_adjNF_H_max;tmp_adjNF_H_max;     nan(1,length(time))];
        Q_adjNF_H_min   = [Q_adjNF_H_min;tmp_adjNF_H_min;     nan(1,length(time))];
        % GSFC firn correction applied
        Q_adjGSFC         = [Q_adjGSFC;tmp_adjGSFC;                 nan(1,length(time))];
        Q_adjGSFC_vel_max = [Q_adjGSFC_vel_max;tmp_adjGSFC_vel_max; nan(1,length(time))];
        Q_adjGSFC_vel_min = [Q_adjGSFC_vel_min;tmp_adjGSFC_vel_min; nan(1,length(time))];
        Q_adjGSFC_H_max   = [Q_adjGSFC_H_max;tmp_adjGSFC_H_max;     nan(1,length(time))];
        Q_adjGSFC_H_min   = [Q_adjGSFC_H_min;tmp_adjGSFC_H_min;     nan(1,length(time))];
    end
    
end
clear tmp* sect*

% Errors
%   - raw
Q_raw_vel_err = ((Q_raw_vel_max-Q_raw) + (Q_raw-Q_raw_vel_min))./2;
Q_raw_H_err = ((Q_raw_H_max-Q_raw) + (Q_raw-Q_raw_H_min))./2;
Q_raw_err = sqrt(Q_raw_vel_err.^2 + Q_raw_H_err.^2); % assume velocity and thickness errors are independent
%   - adj
Q_adj_vel_err = ((Q_adj_vel_max-Q_adj) + (Q_adj-Q_adj_vel_min))./2;
Q_adj_H_err = ((Q_adj_H_max-Q_adj) + (Q_adj-Q_adj_H_min))./2;
Q_adj_err = sqrt(Q_adj_vel_err.^2 + Q_adj_H_err.^2); % assume velocity and thickness errors are independent
%   - BedMachine
Q_BM_vel_err = ((Q_BM_vel_max-Q_BM) + (Q_BM-Q_BM_vel_min))./2;
Q_BM_H_err = ((Q_BM_H_max-Q_BM) + (Q_BM-Q_BM_H_min))./2;
Q_BM_err = sqrt(Q_BM_vel_err.^2 + Q_BM_H_err.^2); % assume velocity and thickness errors are independent
%   - BedMap2
Q_B2_vel_err = ((Q_B2_vel_max-Q_B2) + (Q_B2-Q_B2_vel_min))./2;
Q_B2_H_err = ((Q_B2_H_max-Q_B2) + (Q_B2-Q_B2_H_min))./2;
Q_B2_err = sqrt(Q_B2_vel_err.^2 + Q_B2_H_err.^2); % assume velocity and thickness errors are independent
%   - adj (no thickness corrections applied)
Q_adjS_vel_err = ((Q_adjS_vel_max-Q_adjS) + (Q_adjS-Q_adjS_vel_min))./2;
Q_adjS_H_err = ((Q_adjS_H_max-Q_adjS) + (Q_adjS-Q_adjS_H_min))./2;
Q_adjS_err = sqrt(Q_adjS_vel_err.^2 + Q_adjS_H_err.^2); % assume velocity and thickness errors are independent
%   - adj (no downstream correction)
Q_adjIS_vel_err = ((Q_adjIS_vel_max-Q_adjIS) + (Q_adjIS-Q_adjIS_vel_min))./2;
Q_adjIS_H_err = ((Q_adjIS_H_max-Q_adjIS) + (Q_adjIS-Q_adjIS_H_min))./2;
Q_adjIS_err = sqrt(Q_adjIS_vel_err.^2 + Q_adjIS_H_err.^2); % assume velocity and thickness errors are independent
%   - adj (no dH correction)
Q_adjNDH_vel_err = ((Q_adjNDH_vel_max-Q_adjNDH) + (Q_adjNDH-Q_adjNDH_vel_min))./2;
Q_adjNDH_H_err = ((Q_adjNDH_H_max-Q_adjNDH) + (Q_adjNDH-Q_adjNDH_H_min))./2;
Q_adjNDH_err = sqrt(Q_adjNDH_vel_err.^2 + Q_adjNDH_H_err.^2); % assume velocity and thickness errors are independent
%   - adj (no firn correction)
Q_adjNF_vel_err = ((Q_adjNF_vel_max-Q_adjNF) + (Q_adjNF-Q_adjNF_vel_min))./2;
Q_adjNF_H_err = ((Q_adjNF_H_max-Q_adjNF) + (Q_adjNF-Q_adjNF_H_min))./2;
Q_adjNF_err = sqrt(Q_adjNF_vel_err.^2 + Q_adjNF_H_err.^2); % assume velocity and thickness errors are independent
%   - adj (GSFC firn correction)
Q_adjGSFC_vel_err = ((Q_adjGSFC_vel_max-Q_adjGSFC) + (Q_adjGSFC-Q_adjGSFC_vel_min))./2;
Q_adjGSFC_H_err = ((Q_adjGSFC_H_max-Q_adjGSFC) + (Q_adjGSFC-Q_adjGSFC_H_min))./2;
Q_adjGSFC_err = sqrt(Q_adjGSFC_vel_err.^2 + Q_adjGSFC_H_err.^2); % assume velocity and thickness errors are independent
clear *vel_err *_H_err

%% Save

filled = V.filled;
save(['discharge_timeseries_migrate_' num2str(migration_distance) 'm_SID.mat'],...
    'Q_raw','Q_raw_err',...
    'Q_adj','Q_adj_err',...
    'Q_BM','Q_BM_err',...
    'Q_B2','Q_B2_err',...
    'Q_adjIS','Q_adjIS_err',...
    'Q_adjS','Q_adjS_err',...
    'Q_adjNF','Q_adjNF_err',...
    'Q_adjNDH','Q_adjNDH_err',...
    'Q_adjGSFC','Q_adjGSFC_err',...
    'time','filled');


%% Calculate timeseries for each basin

% basin info
load('balance_discharge/all_basins.mat');

% Pre-allocate
%   - discharge
bQ_raw = zeros([numel(basins.name),length(time)]).*NaN;
bQ_adj = zeros([numel(basins.name),length(time)]).*NaN;
bQ_BM  = zeros([numel(basins.name),length(time)]).*NaN;
bQ_B2  = zeros([numel(basins.name),length(time)]).*NaN;
bQE_raw = zeros([numel(basins.name),length(time)]).*NaN;
bQE_adj = zeros([numel(basins.name),length(time)]).*NaN;
bQE_BM  = zeros([numel(basins.name),length(time)]).*NaN;
bQE_B2  = zeros([numel(basins.name),length(time)]).*NaN;
bQ_adjIS  = zeros([numel(basins.name),length(time)]).*NaN;
bQ_adjS   = zeros([numel(basins.name),length(time)]).*NaN;
bQ_adjNF  = zeros([numel(basins.name),length(time)]).*NaN;
bQ_adjNDH = zeros([numel(basins.name),length(time)]).*NaN;
bQE_adjIS  = zeros([numel(basins.name),length(time)]).*NaN;
bQE_adjS   = zeros([numel(basins.name),length(time)]).*NaN;
bQE_adjNF  = zeros([numel(basins.name),length(time)]).*NaN;
bQE_adjNDH = zeros([numel(basins.name),length(time)]).*NaN;
bQ_adjGSFC  = zeros([numel(basins.name),length(time)]).*NaN;
bQE_adjGSFC = zeros([numel(basins.name),length(time)]).*NaN;
%   - proportion of discharge observed
bQp_raw = zeros([numel(basins.name),length(time)]).*NaN;
bQp_adj = zeros([numel(basins.name),length(time)]).*NaN;
bQp_BM  = zeros([numel(basins.name),length(time)]).*NaN;
bQp_B2  = zeros([numel(basins.name),length(time)]).*NaN;
bQp_adjIS  = zeros([numel(basins.name),length(time)]).*NaN;
bQp_adjS   = zeros([numel(basins.name),length(time)]).*NaN;
bQp_adjNF  = zeros([numel(basins.name),length(time)]).*NaN;
bQp_adjNDH = zeros([numel(basins.name),length(time)]).*NaN;
bQp_adjGSFC = zeros([numel(basins.name),length(time)]).*NaN;

% Make copies where filled discharge is set to nan
filled = logical(filled);
Qf_raw = Q_raw;
Qf_raw(filled) = NaN;
Qf_adj = Q_adj;
Qf_adj(filled) = NaN;
Qf_BM = Q_BM;
Qf_BM(filled) = NaN;
Qf_B2 = Q_B2;
Qf_B2(filled) = NaN;
Qf_adjIS = Q_adjIS;
Qf_adjIS(filled) = NaN;
Qf_adjS = Q_adjS;
Qf_adjS(filled) = NaN;
Qf_adjNF = Q_adjNF;
Qf_adjNF(filled) = NaN;
Qf_adjNDH = Q_adjNDH;
Qf_adjNDH(filled) = NaN;
Qf_adjGSFC = Q_adjGSFC;
Qf_adjGSFC(filled) = NaN;

% calculate discharge
for i = 1:numel(basins.name)
    tf = inpoly2([gate_x(:),gate_y(:)],[basins.x{i},basins.y{i}]);
    if i == 1
        tf = logical(ones([length(gate_x),1]));
    end
    % Discharge
    %   - main
    bQ_raw(i,:)  = nansum(Q_raw(tf,:),1);
    bQE_raw(i,:) = nansum(Q_raw_err(tf,:),1);
    bQ_adj(i,:)  = nansum(Q_adj(tf,:),1);
    bQE_adj(i,:) = nansum(Q_adj_err(tf,:),1);
    bQ_BM(i,:)   = nansum(Q_BM(tf,:),1);
    bQE_BM(i,:)  = nansum(Q_BM_err(tf,:),1);
    bQ_B2(i,:)   = nansum(Q_B2(tf,:),1);
    bQE_B2(i,:)  = nansum(Q_B2_err(tf,:),1);
    %   - corrections
    bQ_adjIS(i,:)  = nansum(Q_adjIS(tf,:),1);
    bQE_adjIS(i,:) = nansum(Q_adjIS_err(tf,:),1);
    bQ_adjS(i,:)  = nansum(Q_adjS(tf,:),1);
    bQE_adjS(i,:) = nansum(Q_adjS_err(tf,:),1);
    bQ_adjNF(i,:)  = nansum(Q_adjNF(tf,:),1);
    bQE_adjNF(i,:) = nansum(Q_adjNF_err(tf,:),1);
    bQ_adjNDH(i,:)  = nansum(Q_adjNDH(tf,:),1);
    bQE_adjNDH(i,:) = nansum(Q_adjNDH_err(tf,:),1);
    %   - alternative firn models
    bQ_adjGSFC(i,:)  = nansum(Q_adjGSFC(tf,:),1);
    bQE_adjGSFC(i,:) = nansum(Q_adjGSFC_err(tf,:),1);
    
    % Proportion of discharge observed
    %   - main
    bQp_raw(i,:) = nansum(Qf_raw(tf,:),1)./bQ_raw(i,:).*100;
    bQp_adj(i,:) = nansum(Qf_adj(tf,:),1)./bQ_adj(i,:).*100;
    bQp_BM(i,:)  = nansum(Qf_BM(tf,:),1)./bQ_BM(i,:).*100;
    bQp_B2(i,:)  = nansum(Qf_B2(tf,:),1)./bQ_B2(i,:).*100;
    %   - corrections
    bQp_adjIS(i,:) = nansum(Qf_adjIS(tf,:),1)./bQ_adjIS(i,:).*100;
    bQp_adjS(i,:) = nansum(Qf_adjS(tf,:),1)./bQ_adjS(i,:).*100;
    bQp_adjNF(i,:) = nansum(Qf_adjNF(tf,:),1)./bQ_adjNF(i,:).*100;
    bQp_adjNDH(i,:) = nansum(Qf_adjNDH(tf,:),1)./bQ_adjNDH(i,:).*100;
    %   - alternative firn models
    bQp_adjGSFC(i,:) = nansum(Qf_adjGSFC(tf,:),1)./bQ_adjGSFC(i,:).*100;
end

% Save
save(['basin_discharge_migrate_' num2str(migration_distance) 'm_SID.mat'],...
    'bQ_raw','bQE_raw','bQp_raw',...
    'bQ_adj','bQE_adj','bQp_adj',...
    'bQ_BM','bQE_BM','bQp_BM',...
    'bQ_B2','bQE_B2','bQp_B2',...
    'bQ_adjIS','bQE_adjIS','bQp_adjIS',...
    'bQ_adjS','bQE_adjS','bQp_adjS',...
    'bQ_adjNF','bQE_adjNF','bQp_adjNF',...
    'bQ_adjNDH','bQE_adjNDH','bQp_adjNDH',...
    'bQ_adjGSFC','bQE_adjGSFC','bQp_adjGSFC',...
    'time');


