% extract_ice_shelf_submarine_melt_noel

clear
close all
clc

% Add paths and libraries
if ispc
    addpath(genpath('D:/work/Fellowship_Leeds/Data'));
    addpath(genpath('../../v2'));
else
    addpath(genpath('/resstore/b0133/Data/MATLAB_tools'));
    addpath(genpath('/resstore/b0133/Data/DEMs/Antarctica/BedMachine'));
    addpath(genpath('/resstore/b0133/Data/ice_shelf_basal_melt_rates/Adusumilli_2020'));
    addpath(genpath('/resstore/b0133/Data/ice_shelf_basal_melt_rates/ITSLIVE_annual_ice_shelf_height_change_basal_melt'));
    addpath(genpath('../../v2'));
end

% Make output directory
mkdir individual_shelf_submarine_melt

%% User options

iceDensity = 917; % kg/m3

% Options
fill_gaps = 0; % legacy from old code that used bedmachine to fill gaps (not doing for now)

%% Load gates

% Load GL structure - contains name and xy coordinates of all shelf
% grounding lines
load('merged_shelf_GLs_all.mat','GL');

%% Load basal melt data from paolo

% load paolo data
fn = 'ANT_G1920V01_IceShelfMelt.nc';
Mx = ncread(fn,'x');
My = ncread(fn,'y');
M = ncread(fn,'melt');
ptime = ncread(fn,'time') + datenum(1950,1,1);
[MX,MY] = meshgrid(Mx,My);

% Estimate balance melt rate
h = ncread(fn,'height_change');
fac = ncread(fn,'fac');
smb = ncread(fn,'smb');
H = (h-fac).*9.3; % convert measured surface height anomaly into thickness anomaly
%   - Calculate the thickness rate: 
isf = any(isfinite(H),3); % mask is true wherever there's any data
H2 = cube2rect(H,isf);  % reshapes using Climate Data Toolbox function
dHdt2 = gradient(H2')'*4; % quarterly sampling, so multiplying thickness gradient by four converts to m/yr
dHdt = rect2cube(dHdt2,isf); % gets it back into shape.
%   - Divergence
D = load('divergence_adusumilli_grid.mat');
fn2 = 'bb0448974g_2_1.h5';
Xd = h5read(fn2,'/x')'; 
Yd = flipud(h5read(fn2,'/y')'); 
D.div = interp2(Xd,Yd,D.div,MX,MY);
%   - balance melt rate
Mb = -(0-smb + D.div);


% Assume 10% error for now
M_err = abs(M).*0.1;

% Interpolate gaps in data
for ii = 1:size(M,1)
    for jj = 1:size(M,2)
        ts = squeeze(M(ii,jj,:));
        if sum(isfinite(ts))>length(ts)*0.3
            ts = interp1gap(ts);
            M(ii,jj,:) = ts;
        end
        ts = squeeze(Mb(ii,jj,:));
        if sum(isfinite(ts))>length(ts)*0.3
            ts = interp1gap(ts);
            Mb(ii,jj,:) = ts;
        end
    end
end
M_err(isnan(M_err)) = abs(M(isnan(M_err))).*0.1;
% Fill remaining gaps using time-average
M_avg = nanmean(M,3);
for i = 1:length(ptime)
    M_layer = M(:,:,i);
    Mb_layer = Mb(:,:,i);
    M_err_layer = M_err(:,:,i);
    M_err_layer(isnan(M_layer)) = abs(M_avg(isnan(M_layer))).*0.1;
    M_layer(isnan(M_layer)) = M_avg(isnan(M_layer));
    Mb_layer(isnan(Mb_layer)) = M_avg(isnan(Mb_layer));
    M(:,:,i) = M_layer;
    Mb(:,:,i) = Mb_layer;
    M_err(:,:,i) = M_err_layer;
end

% For some reason, melting is given as negative, so change sign
M = M.*-1;
Mb = Mb.*-1;

% and rotate
tmp = zeros(size(M)).*NaN;
for i = 1:size(M,3)
    dummy = squeeze(M(:,:,i));
    dummy = flipud(rot90(dummy));
    tmp(:,:,i) = dummy;
end
M = tmp;
clear tmp
%   - Mb
tmp = zeros(size(Mb)).*NaN;
for i = 1:size(Mb,3)
    dummy = squeeze(Mb(:,:,i));
    dummy = flipud(rot90(dummy));
    tmp(:,:,i) = dummy;
end
Mb = tmp;
clear tmp
%   - Merr
tmp = zeros(size(M_err)).*NaN;
for i = 1:size(M_err,3)
    dummy = squeeze(M_err(:,:,i));
    dummy = flipud(rot90(dummy));
    tmp(:,:,i) = dummy;
end
M_err = tmp;
clear tmp


%% Extract basal melt rate for each shelf

for i = 1:numel(GL)
    
    % Get shelf info
    tic;
    shelf_name = GL(i).Name;
    gate_x = GL(i).X;
    gate_y = GL(i).Y;
    disp(' ');
    disp(['Starting shelf number: ' num2str(i) ' - ' shelf_name ]);
    
    % Make time variable for this shelf - combine Paolo and Noel if Noel's
    % data are available
    if isfile(['D:/work/Fellowship_Leeds/Data/Ice_shelf_basal_melt_rates/Noel_2022/timeseries/csv_outputs/' shelf_name '-timeseries.csv'])
        ntime = xlsread([shelf_name '-timeseries.csv'],'C2:C122');
        ntime = datenum(ntime,1,1);
        % Make combined time
        keep = find(ptime<min(ntime));
        time = [ptime(keep);ntime];
        ptime_use = ptime(keep);
    else
        time = ptime;
        ptime_use = ptime;
    end
    
    % Get all available coastline times for this shelf
    if isfile(['../ice_shelf_masks_greene/masks/all_years_islands_removed/' shelf_name '_mask_islands_removed_data.mat']) 
        load([shelf_name '_mask_islands_removed_data.mat'],'coast_time_all','shelf_x','shelf_y');
    else
        disp(['No mask available for ' shelf_name ]);
        return
    end
    
    % Pre-allocate submarine melt timeseries for this shelf
    melt_rate_ts = zeros([length(time),1]).*NaN;
    melt_rate_err_ts = zeros([length(time),1]).*NaN;
    balance_melt_rate_ts = zeros([length(time),1]).*NaN;
    A_ts = zeros([length(time),1]).*NaN; % timeseries of shelf area
    melt_source_ts = zeros([length(time),1]).*NaN;
    
    % Loop through paolo melt time, load nearest-in-time mask and integrate smb
    for j = 1:length(ptime_use)
        % Load closest mask available
        time_diff = abs(coast_time_all-ptime_use(j));
        idx = find(time_diff == min(time_diff));
        sx = shelf_x{idx};
        sy = shelf_y{idx};
        
        % If doing Thwaites and the time is 2009 to 2019, use Julia's
        % shapefiles
        if strcmp(shelf_name,'Thwaites') && ptime_use(j)>=datenum(2009,1,1) && ptime_use(j)<=datenum(2019,12,31)
            [current_year,~,~] = datevec(ptime_use(j));
            thw_mask = shaperead(['D:/work/Fellowship_Leeds/Data/Calving_front_locations/Thwaites/Thwaites_ShelfArea/' num2str(current_year) 'tw_area3.shp']);
            sx = thw_mask.X;
            sy = thw_mask.Y;
        end
        
        % Get area of shelf for integration
        [slat,slon] = ps2ll(sx,sy);
        wgs84 = wgs84Ellipsoid("km");
        stats = regionprops(isfinite(slat),'PixelIdxList');
        A = 0;
        for k = 1:length(stats)
            A = A + areaint(slat(stats(k).PixelIdxList),slon(stats(k).PixelIdxList),wgs84); % accounts for curvature of earth
        end
        A = A * (1000*1000); % convert from km^2 to m^2
        %A = area(polyshape(sx,sy)); % m^2
        A_ts(j) = A;
        
        % Get melt pixels in ice shelf
        in = inpoly2([MX(:),MY(:)],[sx(:),sy(:)]);
        
        % Check for shelves with no basal melt coverage
        if sum(in)==0
            % get basal melt closest to centroid of ice shelf
            M_layer = M(:,:,j);
            [cx,cy] = centroid(polyshape(sx,sy));
            d = sqrt((cx-MX(:)).^2 + (cy-MY(:)).^2);
            d(isnan(M_layer(:))) = NaN;
            in = find(d(:) == nanmin(d(:)),1,'first');
        end
        
        % Extract basal melt rate
        melt_layer = M(:,:,j);
        melt_err_layer = M_err(:,:,j);
        Mb_layer = Mb(:,:,j);
        if isnan(nanmean(melt_layer(in))) 
            [cx,cy] = centroid(polyshape(sx,sy));
            d = sqrt((cx-MX(:)).^2 + (cy-MY(:)).^2);
            d(isnan(melt_layer(:))) = NaN;
            in = find(d(:) == nanmin(d(:)),1,'first');
        end
        if isnan(Mb_layer(in))
            Mb_layer(in) = melt_layer(in);
        end
        if isnan(melt_err_layer(in))
            melt_err_layer(in) = abs(melt_layer).*0.1;
        end
        melt_rate_ts(j) = nanmean(melt_layer(in));
        melt_rate_err_ts(j) = nanmean(melt_err_layer(in));
        balance_melt_rate_ts(j) = nanmean(Mb_layer(in));
        melt_source_ts(j) = 2;
    end
    
    % Then loop through noel melt time, if it's available 
    if isfile(['D:/work/Fellowship_Leeds/Data/Ice_shelf_basal_melt_rates/Noel_2022/timeseries/csv_outputs/' shelf_name '-timeseries.csv'])
        melt_count = j;
        nmelt = xlsread([shelf_name '-timeseries.csv'],'D2:D122');
        nmelt_err = xlsread([shelf_name '-timeseries.csv'],'E2:E122');
        nmelt = fillmissing(nmelt,'linear');
        nmelt = fillmissing(nmelt,'Previous');
        nmelt = fillmissing(nmelt,'Next');
        nmelt_err = fillmissing(nmelt_err,'linear');
        nmelt_err = fillmissing(nmelt_err,'Previous');
        nmelt_err = fillmissing(nmelt_err,'Next');
        melt_rate_ts = [melt_rate_ts(1:melt_count) ; nmelt];
        melt_rate_err_ts = [melt_rate_err_ts(1:melt_count) ; nmelt_err];
        balance_melt_rate_ts = [balance_melt_rate_ts(1:melt_count) ; nmelt]; % no balance melt rate data provided yet, so assume balance melt rate here. This is recalculated later based on mass balance.
        % loop through months and get shelf area at each time-step
        for j = 1:length(ntime)
            melt_count = melt_count+1;
            time_diff = abs(coast_time_all-ntime(j));
            idx = find(time_diff == min(time_diff));
            sx = shelf_x{idx};
            sy = shelf_y{idx};
            % If doing Thwaites and the time is 2009 to 2019, use Julia's
            % shapefiles
            if strcmp(shelf_name,'Thwaites') && ntime(j)>=datenum(2009,1,1) && ntime(j)<=datenum(2019,12,31)
                [current_year,~,~] = datevec(ntime(j));
                thw_mask = shaperead(['D:/work/Fellowship_Leeds/Data/Calving_front_locations/Thwaites/Thwaites_ShelfArea/' num2str(current_year) 'tw_area3.shp']);
                sx = thw_mask.X;
                sy = thw_mask.Y;
            end
            A = area(polyshape(sx,sy)); % m^2
            A_ts(melt_count) = A;
            melt_source_ts(melt_count) = 3;
        end
    end
    
    % Calculate melt flux for this shelf
    melt_flux_ts = melt_rate_ts.*iceDensity.*A_ts.*1e-12;
    melt_flux_err_ts = melt_rate_err_ts.*iceDensity.*A_ts.*1e-12;
    balance_melt_flux_ts = balance_melt_rate_ts.*iceDensity.*A_ts.*1e-12;
    
    % Plot melt timeseries for this shelf
    figure('units','inches','outerposition',[5 5 8 3.5],'color','w','visible','off')
    plot(time,melt_flux_ts,'k','linewidth',2); hold on; datetick('x','keeplimits');
    plot(time,melt_flux_ts-melt_flux_err_ts,'k','linewidth',1,'linestyle',':');
    plot(time,melt_flux_ts+melt_flux_err_ts,'k','linewidth',1,'linestyle',':');
    ylabel('Melt flux (Gt yr^{-1})');
    yyaxis right
    plot(time,A_ts./(1000*1000),'b');
    ylabel('Area (km^2)');
    ax=gca;
    ax.YAxis(2).Color = 'b';
    export_fig(['individual_shelf_submarine_melt/' shelf_name '_submarine_melt_timeseries_basic_noel.png'],'-dpng','-r300');
    close
    
    % Save data for this shelf
    save(['individual_shelf_submarine_melt/' shelf_name '_submarine_melt_timeseries_noel.mat'],'time','A_ts',...
        'melt_flux_ts','melt_flux_err_ts','melt_rate_ts','melt_rate_err_ts',...
        'balance_melt_rate_ts','balance_melt_flux_ts','melt_source_ts');
    
end



