% extract_timeseries_dHdt
%
% This script merges the dH timeseries from Nilsson et al (2022) and Paolo
% et al (2022) and calculated 5-year rolling dHdt during their common time
% periods, then does some gap filling before extracting dHdt at my flux
% gates
%
% This just about runs on my laptop, but takes hours

clear
close all
clc

addpath(genpath('D:/work/Fellowship_Leeds/Data'));


%% Calculate dHdt over grounded ice

% load reference thickness error
refH = load('../reference_thickness_200m.mat','H_err','x','y');
[refH.X,refH.Y] = meshgrid(refH.x,refH.y);

% load data
fn = 'ANT_G1920V01_GroundedIceHeight.nc';
x = ncread(fn,'x');
y = ncread(fn,'y');
[X,Y] = meshgrid(x,y);
Herr = interp2(refH.X,refH.Y,refH.H_err,X,Y);
clear refH X Y
time = ncread(fn,'time') + datenum(1950,1,1);
dh = ncread(fn,'dh'); % height anomaly w.r.t 2013-12-16; metres
dhE = ncread(fn,'rmse'); 

% trim to common time period for floating ice
fn2 = 'ANT_G1920V01_IceShelfMelt.nc';
time2 = ncread(fn2,'time') + datenum(1950,1,1);
ind = time>=min(time2) & time<=max(time2);
dh = dh(:,:,ind);
dhE = dhE(:,:,ind);
time = time(ind);
clear time2 fn2 ind

% prepare error calculation
t1 = datetime(time,'convertfrom','datenum');
t2 = dateshift(t1, 'start', 'year');
t3 = dateshift(t1, 'end', 'year');
yrf = year(t1) + (t1-t2)./(t3-t2); % decimal year
clear t1 t2 t3

% calculate dhdt in rolling 5 year intervals - for grounded ice, height anomaly is equivalent to a thickness anomaly
dHdt_ts = zeros(size(dh)).*NaN;
dHdtE_ts = zeros(size(dh)).*NaN;
p_ts = zeros(size(dh)).*NaN;
for i = 1:length(time)
    ind = find(time>=time(i)-365*2.5 & time<=time(i)+365*2.5);
    [dHdt_ts(:,:,i),p_ts(:,:,i)] = trend(dh(:,:,ind),time(ind),'dim',3,'omitnan');
    dHdt_ts(:,:,i) = flipud(rot90(dHdt_ts(:,:,i)));
    p_ts(:,:,i) = flipud(rot90(p_ts(:,:,i)));
    % calculate dHdt error from the trend: Equation 8.17 on pg 188 of Introduction to Error Analysis (Taylor)
    del = sum(yrf(ind))*sum(yrf(ind).^2) - sum(yrf(ind))^2;
    dHdtE_ts(:,:,i) = (Herr+flipud(rot90(nanmean(dhE(:,:,ind),3))).^2).*sqrt(sum(yrf(ind))/del); % dHE.^2 to get total RMS error (Nilsson et al 2022), then add the thickness error(? - otherwise the errors are unrealistically small...) 
end
dHdt_ts = dHdt_ts.*365.25; % convert to metres per year

% Save
save('dHdt_grounded_grid_ts.mat','x','y','dHdt_ts','dHdtE_ts','p_ts','time','-v7.3');
clear dh


%% Calculate dHdt over floating ice

% load data
fn = 'ANT_G1920V01_IceShelfMelt.nc';
x = ncread(fn,'x');
y = ncread(fn,'y');
time = ncread(fn,'time') + datenum(1950,1,1);
H = ncread(fn,'thickness');
Herr = ncread(fn,'thickness_err');

% prepare error calculation
t1 = datetime(time,'convertfrom','datenum');
t2 = dateshift(t1, 'start', 'year');
t3 = dateshift(t1, 'end', 'year');
yrf = year(t1) + (t1-t2)./(t3-t2); % decimal year
clear t1 t2 t3

% calculate dhdt in rolling 5 year intervals
dHdt_ts = zeros(size(H)).*NaN;
dHdtE_ts = zeros(size(H)).*NaN;
p_ts = zeros(size(H)).*NaN;
for i = 1:length(time)
    ind = find(time>=time(i)-365*2.5 & time<=time(i)+365*2.5);
    [dHdt_ts(:,:,i),p_ts(:,:,i)] = trend(H(:,:,ind),time(ind),'dim',3,'omitnan');
    dHdt_ts(:,:,i) = flipud(rot90(dHdt_ts(:,:,i)));
    p_ts(:,:,i) = flipud(rot90(p_ts(:,:,i)));
    % calculate dHdt error from the trend: Equation 8.17 on pg 188 of Introduction to Error Analysis (Taylor)
    del = sum(yrf(ind))*sum(yrf(ind).^2) - sum(yrf(ind))^2;
    dHdtE_ts(:,:,i) = flipud(rot90(nanmean(Herr(:,:,ind),3).*sqrt(sum(yrf(ind))/del)));
end
dHdt_ts = dHdt_ts.*365.25; % convert to metres per year

% Save
save('dHdt_floating_grid_ts.mat','x','y','dHdt_ts','dHdtE_ts','p_ts','time','-v7.3'); % these are huge...


%% Merge floating and grounded grids

clear

% load and merge
grnd = load('dHdt_grounded_grid_ts.mat');
flt = load('dHdt_floating_grid_ts.mat');
dHdt_ts = zeros(size(flt.dHdt_ts)).*NaN;
p_ts = zeros(size(flt.dHdt_ts)).*NaN;
for i = 1:length(flt.time)
    [~,ind] = findNearest(grnd.time,flt.time(i));
    dHdt_ts(:,:,i) = nanmean(cat(3,grnd.dHdt_ts(:,:,ind),flt.dHdt_ts(:,:,i)),3);
    dHdtE_ts(:,:,i) = nanmean(cat(3,grnd.dHdtE_ts(:,:,ind),flt.dHdtE_ts(:,:,i)),3);
    p_ts(:,:,i) = nanmean(cat(3,grnd.p_ts(:,:,ind),flt.p_ts(:,:,i)),3);
end
x = grnd.x;
y = grnd.y;
time = flt.time;
clear grnd flt
% Save merged version without filling
save('dHdt_merged_ts_raw.mat','dHdt_ts','dHdtE_ts','p_ts','x','y','time','-v7.3');

% Fill gaps with time-series information
[r,c,p] = size(dHdt_ts);
filled = isnan(dHdt_ts);
dHdt_ts = cube2rect(dHdt_ts); % reshapes so that each column represents a single pixel through time
dHdt_ts = fillmissing(dHdt_ts,'linear','EndValues','none');
dHdt_ts = fillmissing(dHdt_ts,'Next');
dHdt_ts = fillmissing(dHdt_ts,'Previous');
dHdt_ts = rect2cube(dHdt_ts,[r,c,p]);

% fill internal gaps
for i = 1:length(time)
    L = bwlabel(isnan(dHdt_ts(:,:,i)));
    dHdt_ts(:,:,i) = regionfill(dHdt_ts(:,:,i),L>1);
end

% crudely fill extrapolate to the edges using regionfill, just to make sure
% the dHdt grids cover the gates.
for i = 1:length(time)
    tmp = dHdt_ts(:,:,i);
    tmp([1,end],:) = 0; % constrain region fill by setting edges to 0
    tmp(:,[1,end]) = 0; % constrain region fill by setting edges to 0
    tmp = regionfill(tmp,isnan(tmp));
    dHdt_ts(:,:,i) = tmp;
end

% Fill gaps in p and error
p_ts(filled) = 9999;
dHdtE_ts(filled) = 0.1; % assume 0.1 m/yr error where we have filled gaps

% Save
save('dHdt_merged_ts_filled.mat','dHdt_ts','dHdtE_ts','p_ts','x','y','time','-v7.3');

%% Extract at gate

% Load flux gates
load('../../flux_gates/flux_gates_migrated.mat');
migration_distance = 0:200:3000; % metres
[X,Y] = meshgrid(x,y);
gate_dHdt_ts = cell([numel(gate_x_migrate),1]);
gate_dHdtE_ts = cell([numel(gate_x_migrate),1]);
gate_p_ts = cell([numel(gate_x_migrate),1]);

% loop through gates
for i = 1:numel(gate_x_migrate)
    % get gate
    gate_x = gate_x_migrate{i};
    gate_y = gate_y_migrate{i};
    % pre-allocate timeseries for this gate
    tmp_dHdt = zeros([length(gate_x),length(time)]).*NaN;
    tmp_dHdtE = zeros([length(gate_x),length(time)]).*NaN;
    tmp_p = zeros([length(gate_x),length(time)]).*NaN;
    % interpolate to gate
    for j = 1:length(time)
        tmp_dHdt(:,j) = interp2(X,Y,dHdt_ts(:,:,j),gate_x,gate_y);
        tmp_dHdtE(:,j) = interp2(X,Y,dHdtE_ts(:,:,j),gate_x,gate_y);
        tmp_p(:,j) = interp2(X,Y,p_ts(:,:,j),gate_x,gate_y);
    end
    % build
    gate_dHdt_ts{i} = tmp_dHdt;
    gate_dHdtE_ts{i} = tmp_dHdtE;
    gate_p_ts{i} = tmp_p;
end
% save
save('gate_dHdt_ts.mat','gate_dHdt_ts','gate_dHdtE_ts','gate_p_ts','time','migration_distance','-v7.3');


% %% Extend timeseries just with grounded ice dHdt? I thought about doing
% this to go back to 1985, but never quite finished it
% 
% % load merged time
% M = load('dHdt_merged_ts_filled.mat','time');
% 
% % load full grounded dataset
% grnd = load('dHdt_grounded_grid_ts.mat');
% 
% % trim grounded dataset to just before and after merged time
% ind = find(grnd.time<min(M.time) | grnd.time>max(M.time));
% grnd.dHdt_ts = grnd.dHdt_ts(:,:,ind);
% grnd.p_ts = grnd.p_ts(:,:,ind);
% grnd.time = grnd.time(ind);
% 
% % merge grounded and floating
% flt = load('dHdt_floating_grid_ts.mat','dHdt_ts');
% 
% 
% % fill internal gaps
% for i = 1:length(grnd.time)
%     L = bwlabel(isnan(grnd.dHdt_ts(:,:,i)));
%     grnd.dHdt_ts(:,:,i) = regionfill(grnd.dHdt_ts(:,:,i),L>1);
% end
% 
% % crudely fill external gaps using regionfill
% for i = 1:length(grnd.time)
%     tmp = grnd.dHdt_ts(:,:,i);
%     tmp([1,end],:) = 0; % constrain region fill by setting edges to 0
%     tmp(:,[1,end]) = 0; % constrain region fill by setting edges to 0
%     tmp = regionfill(tmp,isnan(tmp));
%     grnd.dHdt_ts(:,:,i) = tmp;
% end
