% integrate_ice_shelf_SMB_ensemble

clear
close all
clc

% addpaths to data and libraries
if ispc
    addpath(genpath('D:/work/Fellowship_Leeds/Data'));
else
    addpath(genpath('/resstore/b0133/Data/MATLAB_tools'))
    addpath(genpath('/resstore/b0133/Data/RACMO'));
end
addpath(genpath('../grounding_line_discharge'));
addpath(genpath('../ice_shelf_masks_greene'));
addpath(genpath('../ice_shelf_masks'));

% make output dirs
mkdir individual_shelf_SMB_timeseries_ensemble


%% User options

% make plots?
do_plotting=0;

%% Load gates

% Load GL structure - contains name and xy coordinates of all shelf
% grounding lines
load('GL_consolidated_200m.mat');

%% Get area of each ice shelf through time

disp('Calculating ice shelf area...');

% Get coastline times
load([GL(1).Name '_mask_islands_removed_data.mat'],'coast_time_all');

% pre-allocate
A_all = zeros([numel(GL),length(coast_time_all)]).*NaN;
sx_all = cell([numel(GL),length(coast_time_all)]);
sy_all = cell([numel(GL),length(coast_time_all)]);

% loop through ice shelves
for i = 1:numel(GL)
    
    disp(['Processing ice shelf ' num2str(i) ': ' GL(i).Name ]);
    
    % Get shelf info
    shelf_name = GL(i).Name;
    gate_x = GL(i).X;
    gate_y = GL(i).Y;
    
    % Get all available coastline times for this shelf
    if isfile(['../ice_shelf_masks_greene/masks/all_years_islands_removed_consolidated/' shelf_name '_mask_islands_removed_data.mat']) 
        load([shelf_name '_mask_islands_removed_data.mat']);
    else
        disp(['No mask available for ' shelf_name]);
        return
    end
    
    % Loop through SMB time, load nearest-in-time mask and integrate smb
    for j = 1:length(coast_time_all)
        
        % Load closest mask available
        sx = shelf_x{j};
        sy = shelf_y{j};
        sx_all{i,j} = sx;
        sy_all{i,j} = sy;
        [slat,slon] = ps2ll(sx,sy);
        
        % Get area of shelf for integration
        wgs84 = wgs84Ellipsoid("km");
        stats = regionprops(isfinite(slat),'PixelIdxList');
        A = 0;
        for k = 1:length(stats)
            A = A + areaint(slat(stats(k).PixelIdxList),slon(stats(k).PixelIdxList),wgs84); % accounts for curvature of earth
        end
        A = A * (1000*1000); % convert from km^2 to m^2
        %A = area(polyshape(sx,sy)); % m^2
        A_all(i,j) = A;
        
    end
end
clearvars -except A_all sx_all sy_all GL do_plotting coast_time_all shelf_names


%% Get MAR timeseries for each ice shelf

disp(' ');
disp('Extracting MAR timeseries...');

% Load MAR monthly SMB
smb = ncread('mon-SMB-MAR_ERA5-1980-2021.nc','SMB');
x = ncread('mon-SMB-MAR_ERA5-1980-2021.nc','X');
y = ncread('mon-SMB-MAR_ERA5-1980-2021.nc','Y');
MAR_time = double(ncread('mon-SMB-MAR_ERA5-1980-2021.nc','TIME') + datenum(1977,3,1));
smb = squeeze(smb(:,:,1,:));
smb(smb==0)=NaN;

% set x and y to units metres
x = x.*1000;
y = y.*1000;
[X,Y] = meshgrid(x,y);

% Flip and rotate
smb_dummy = smb;
smb = zeros([size(smb,2),size(smb,1),size(smb,3)]).*NaN;
for i = 1:size(smb,3)
    smb(:,:,i) = flipud(rot90(smb_dummy(:,:,i)));
end
clear smb_dummy

% climatology grid
idx = find(MAR_time>=datenum(1979,1,1) & MAR_time<datenum(2009,1,1));
smb_avg = nanmean(smb(:,:,idx),3);

% Get multiple climatologies for uncertainty
start_times = datenum(1979:1989,1,1);
end_times = datenum(1999:2009,1,1);
smb_all_climatologies = cell([length(start_times),1]);
for i = 1:length(start_times)
    idx = find(mean(MAR_time,2)>=start_times(i) & mean(MAR_time,2)<end_times(i));
    smb_tmp = nanmean(smb(:,:,idx),3);
    nanval = smb_tmp(1);
    smb_tmp(smb_tmp==nanval) = NaN;
    smb_all_climatologies{i} = smb_tmp;
end

% pre-allocate output
MAR_all = zeros([numel(GL),length(MAR_time)]).*NaN;
MAR_clim_all = zeros([numel(GL),length(MAR_time)]).*NaN;
MAR_clim_err_all = zeros([numel(GL),length(MAR_time)]).*NaN; 

% Loop through ice shelves
for i = 1:numel(GL)
    
    disp(['Processing ice shelf ' num2str(i) ': ' GL(i).Name ]);
    
    % Loop through MAR time
    for j = 1:length(MAR_time)
        
        % Get coast time closest to this MAR time
        time_diff = abs(coast_time_all-MAR_time(j));
        idx = find(time_diff == min(time_diff));
        
        % keep track of idx, so we only repeat inpoly2 if using a new
        % coastline
        if j == 1
            prev_idx = 0;
        end
            
        if idx ~= prev_idx
            % Pull out ice shelf coordinates
            sx = sx_all{i,idx};
            sy = sy_all{i,idx};
            % Get smb_cells in ice shelf
            [in,on] = inpoly2([X(:),Y(:)],[sx,sy],[],eps^0.2);
            in(on==1) = 1;
            % Check for shelves with no SMB coverage
            smb_layer = smb(:,:,j);
            nan_val = smb_layer(1,1);
            smb_layer(smb_layer==nan_val) = NaN;
            if sum(in)==0 || sum(isfinite(smb_layer(in)))==0
                % get SMB closest to centroid of ice shelf
                nan_val = smb_layer(1,1);
                smb_layer(smb_layer==nan_val) = NaN;
                [cx,cy] = centroid(polyshape(sx,sy));
                d = sqrt((cx-X).^2 + (cy-Y).^2);
                d(isnan(smb_layer(:))) = NaN;
                in = find(d(:) == nanmin(d(:)),1,'first');
            end
            % reset previous idx
            prev_idx = idx;
        end

        % Extract SMB
        smb_layer = smb(:,:,j);
        %   - get number of days in month for unit conversion
        [Yr,M,~] = datevec(MAR_time(j));
        E = eomday(Yr,M);
        % Extract data in each basin
        MAR_all(i,j) = nanmean(smb_layer(in))*A_all(i,idx)*1e-12/E*365.25; % Gt/yr
        MAR_clim_all(i,j) = nanmean(smb_avg(in))*A_all(i,idx)*1e-12/E*365.25; % Gt/yr
        
        % Loop through different climatologies
        MAR_clim_range = zeros([numel(smb_all_climatologies),1]).*NaN;
        for k = 1:numel(smb_all_climatologies)
            smb_tmp = smb_all_climatologies{k};
            nan_val = smb_tmp(1,1);
            smb_tmp(smb_tmp==nan_val) = NaN;
            if sum(in)==0 || sum(isfinite(smb_tmp(in)))==0
                % get SMB closest to centroid of ice shelf
                nan_val = smb_tmp(1,1);
                smb_tmp(smb_tmp==nan_val) = NaN;
                [cx,cy] = centroid(polyshape(sx,sy));
                d = sqrt((cx-X).^2 + (cy-Y).^2);
                d(isnan(smb_tmp(:))) = NaN;
                in = find(d(:) == nanmin(d(:)),1,'first');
            end
            MAR_clim_range(k) = nanmean(smb_tmp(in))*A_all(i,idx)*1e-12/E*365.25; % Gt/yr
        end
        MAR_clim_err_all(i,j) = rssq(MAR_clim_range-MAR_clim_all(i,j));
        
    end
end

% clear some space
clear MAR_clim_range time_diff idx prev_idx sx sy in on smb_layer nan_val cx cy d Yr M E X Y



%% Get HIRHAM timeseries for each ice shelf

disp(' ');
disp('Extracting HIRHAM timeseries...');

% Get list of all HIRHAM files
filelist = [dir('D:\work\Fellowship_Leeds\Data\HIRHAM/ANT22_trans-Daily2D_1979-1989/*.nc') ;...
    dir('D:\work\Fellowship_Leeds\Data\HIRHAM/ANT22_trans-Daily2D_1990-1999/*.nc');...
    dir('D:\work\Fellowship_Leeds\Data\HIRHAM/ANT22_trans-Daily2D_2000-2009/*.nc');...
    dir('D:\work\Fellowship_Leeds\Data\HIRHAM/ANT22_trans-Daily2D_2010-2017/*.nc');...
    dir('D:\work\Fellowship_Leeds\Data\HIRHAM/ANT22_trans-Daily2D_2018/*.nc')];
fn = filelist(1).name;

% Subset over Ronne
lat = ncread(fn,'lat');
lon = ncread(fn,'lon');
[X,Y] = ll2ps(lat,lon);

% Set time (daily from 1979,1,1 to 2018,12,31
HIRHAM_time = datenum(1979,1,1):datenum(2018,12,31);
HIRHAM_monthly_time = [datenum(1979,1:480,1) ; datenum(1979,2:481,1)-1]';

% pre-allocate monthly output
smb = zeros([size(lat),length(HIRHAM_monthly_time)]).*NaN;

% Load; Units are mm w.e. per day
for i = 1:size(HIRHAM_monthly_time,1)
    idx = find(HIRHAM_time>=HIRHAM_monthly_time(i,1) & HIRHAM_time<=HIRHAM_monthly_time(i,2));
    smb_monthly_stack = zeros([size(lat),length(idx)]).*NaN;
    for j = 1:length(idx)
        smb_monthly_stack(:,:,j) = ncread([filelist(idx(j)).folder '/' filelist(idx(j)).name],'gld');
    end
    smb_monthly_stack(smb_monthly_stack==-9999) = NaN;
    smb(:,:,i) = nanmean(smb_monthly_stack,3);
end
clear smb_monthly_stack idx

% climatology grid
idx = find(nanmean(HIRHAM_monthly_time,2)>=datenum(1979,1,1) & nanmean(HIRHAM_monthly_time,2)<datenum(2009,1,1));
smb_avg = nanmean(smb(:,:,idx),3);

% Get multiple climatologies for uncertainty
start_times = datenum(1979:1989,1,1);
end_times = datenum(1999:2009,1,1);
smb_all_climatologies = cell([length(start_times),1]);
for i = 1:length(start_times)
    idx = find(mean(HIRHAM_monthly_time,2)>=start_times(i) & mean(HIRHAM_monthly_time,2)<end_times(i));
    smb_tmp = nanmean(smb(:,:,idx),3);
    nanval = smb_tmp(1);
    smb_tmp(smb_tmp==nanval) = NaN;
    smb_all_climatologies{i} = smb_tmp;
end

% pre-allocate output
HIRHAM_all = zeros([numel(GL),length(HIRHAM_monthly_time)]).*NaN;
HIRHAM_clim_all = zeros([numel(GL),length(HIRHAM_monthly_time)]).*NaN;
HIRHAM_clim_err_all = zeros([numel(GL),length(HIRHAM_monthly_time)]).*NaN;

% Loop through ice shelves
for i = 1:numel(GL)
    
    disp(['Processing ice shelf ' num2str(i) ': ' GL(i).Name ]);
    
    % Loop through HIRHAM time
    for j = 1:length(HIRHAM_monthly_time)
        
        % Get coast time closest to this HIRHAM time
        time_diff = abs(coast_time_all-HIRHAM_monthly_time(j));
        idx = find(time_diff == min(time_diff));
        
        % keep track of idx, so we only repeat inpoly2 if using a new
        % coastline
        if j == 1
            prev_idx = 0;
        end
            
        if idx ~= prev_idx
            % Pull out ice shelf coordinates
            sx = sx_all{i,idx};
            sy = sy_all{i,idx};
            % Get smb_cells in ice shelf
            [in,on] = inpoly2([X(:),Y(:)],[sx,sy],[],eps^0.2);
            in(on==1) = 1;
            % Check for shelves with no SMB coverage
            smb_layer = smb(:,:,j);
            nan_val = smb_layer(1,1);
            smb_layer(smb_layer==nan_val) = NaN;
            if sum(in)==0 || sum(isfinite(smb_layer(in)))==0
                % get SMB closest to centroid of ice shelf
                nan_val = smb_layer(1,1);
                smb_layer(smb_layer==nan_val) = NaN;
                [cx,cy] = centroid(polyshape(sx,sy));
                d = sqrt((cx-X).^2 + (cy-Y).^2);
                d(isnan(smb_layer(:))) = NaN;
                in = find(d(:) == nanmin(d(:)),1,'first');
            end
            % reset previous idx
            prev_idx = idx;
        end

        % Extract SMB
        smb_layer = smb(:,:,j);
        % Extract data in each basin
        HIRHAM_all(i,j) = nanmean(smb_layer(in))*A_all(i,idx)*1e-12*365.25; % Gt/yr
        HIRHAM_clim_all(i,j) = nanmean(smb_avg(in))*A_all(i,idx)*1e-12*365.25; % Gt/yr
        
        % Loop through different climatologies
        HIRHAM_clim_range = zeros([numel(smb_all_climatologies),1]).*NaN;
        for k = 1:numel(smb_all_climatologies)
            smb_tmp = smb_all_climatologies{k};
            nan_val = smb_tmp(1,1);
            smb_tmp(smb_tmp==nan_val) = NaN;
            if sum(in)==0 || sum(isfinite(smb_tmp(in)))==0
                % get SMB closest to centroid of ice shelf
                nan_val = smb_tmp(1,1);
                smb_tmp(smb_tmp==nan_val) = NaN;
                [cx,cy] = centroid(polyshape(sx,sy));
                d = sqrt((cx-X).^2 + (cy-Y).^2);
                d(isnan(smb_tmp(:))) = NaN;
                in = find(d(:) == nanmin(d(:)),1,'first');
            end
            HIRHAM_clim_range(k) = nanmean(smb_tmp(in))*A_all(i,idx)*1e-12*365.25; % Gt/yr
        end
        HIRHAM_clim_err_all(i,j) = rssq(HIRHAM_clim_range-HIRHAM_clim_all(i,j));
        
    end
end

% clear some space
HIRHAM_time = nanmean(HIRHAM_monthly_time,2);
clear HIRHAM_clim_range filelist fn lat lon X Y HIRHAM_monthly_time idx time_diff prev_idx sx sy in on smb_layer nan_val cx cy d 

%% Get RACMO timeseries for each ice shelf

disp(' ');
disp('Extracting RACMO timeseries...');

% Load basic info
fn = 'RACMO2.3p2_ERA5_ANT27_smb_monthly_1979_01_2021_12.nc';
RACMO_time = datenum(1950,1,1) + ncread(fn,'time');
lat = ncread(fn,'lat');
lon = ncread(fn,'lon');
[X,Y] = ll2ps(lat,lon); % requires Antarctic Mapping Tools for MATLAB

% Load
smb = ncread(fn,'smb');
smb = permute(smb,[1,2,4,3]); % deal with odd dimension

% climatology grid
idx = find(RACMO_time>=datenum(1979,1,1) & RACMO_time<datenum(2009,1,1));
smb_avg = nanmean(smb(:,:,idx),3);

% Get multiple climatologies for uncertainty
start_times = datenum(1979:1989,1,1);
end_times = datenum(1999:2009,1,1);
smb_all_climatologies = cell([length(start_times),1]);
for i = 1:length(start_times)
    idx = find(mean(RACMO_time,2)>=start_times(i) & mean(RACMO_time,2)<end_times(i));
    smb_tmp = nanmean(smb(:,:,idx),3);
    nanval = smb_tmp(1);
    smb_tmp(smb_tmp==nanval) = NaN;
    smb_all_climatologies{i} = smb_tmp;
end

% pre-allocate output
RACMO_all = zeros([numel(GL),length(RACMO_time)]).*NaN;
RACMO_clim_all = zeros([numel(GL),length(RACMO_time)]).*NaN;
RACMO_clim_err_all = zeros([numel(GL),length(RACMO_time)]).*NaN;
A_all_use = zeros([numel(GL),length(RACMO_time)]).*NaN;

% Loop through ice shelves
for i = 1:numel(GL)
    
    disp(['Processing ice shelf ' num2str(i) ': ' GL(i).Name ]);
    
    % Loop through RACMO time
    for j = 1:length(RACMO_time)
        
        % Get coast time closest to this RACMO time
        time_diff = abs(coast_time_all-RACMO_time(j));
        idx = find(time_diff == min(time_diff));
        
        % keep track of idx, so we only repeat inpoly2 if using a new
        % coastline
        if j == 1
            prev_idx = 0;
        end
            
        if idx ~= prev_idx
            % Pull out ice shelf coordinates
            sx = sx_all{i,idx};
            sy = sy_all{i,idx};
            % Get smb_cells in ice shelf
            [in,on] = inpoly2([X(:),Y(:)],[sx,sy],[],eps^0.2);
            in(on==1) = 1;
            % Check for shelves with no SMB coverage
            smb_layer = smb(:,:,j);
            nan_val = smb_layer(1,1);
            smb_layer(smb_layer==nan_val) = NaN;
            if sum(in)==0 || sum(isfinite(smb_layer(in)))==0
                % get SMB closest to centroid of ice shelf
                nan_val = smb_layer(1,1);
                smb_layer(smb_layer==nan_val) = NaN;
                [cx,cy] = centroid(polyshape(sx,sy));
                d = sqrt((cx-X).^2 + (cy-Y).^2);
                d(isnan(smb_layer(:))) = NaN;
                in = find(d(:) == nanmin(d(:)),1,'first');
            end
            % reset previous idx
            prev_idx = idx;
        end

        % Extract SMB
        smb_layer = smb(:,:,j);
        nan_val = smb_layer(1,1);
        smb_layer(smb_layer==nan_val) = NaN;
        %   - get number of days in month for unit conversion
        [Yr,M,~] = datevec(RACMO_time(j));
        E = eomday(Yr,M);
        % Extract data in each basin
        RACMO_all(i,j) = nanmean(smb_layer(in))*A_all(i,idx)*1e-12/E*365.25; % Gt/yr
        RACMO_clim_all(i,j) = nanmean(smb_avg(in))*A_all(i,idx)*1e-12/E*365.25; % Gt/yr
        A_all_use(i,j) = A_all(i,idx);
        
        % Loop through different climatologies
        RACMO_clim_range = zeros([numel(smb_all_climatologies),1]).*NaN;
        for k = 1:numel(smb_all_climatologies)
            smb_tmp = smb_all_climatologies{k};
            nan_val = smb_tmp(1,1);
            smb_tmp(smb_tmp==nan_val) = NaN;
            if sum(in)==0 || sum(isfinite(smb_tmp(in)))==0
                % get SMB closest to centroid of ice shelf
                nan_val = smb_tmp(1,1);
                smb_tmp(smb_tmp==nan_val) = NaN;
                [cx,cy] = centroid(polyshape(sx,sy));
                d = sqrt((cx-X).^2 + (cy-Y).^2);
                d(isnan(smb_tmp(:))) = NaN;
                in = find(d(:) == nanmin(d(:)),1,'first');
            end
            RACMO_clim_range(k) = nanmean(smb_tmp(in))*A_all(i,idx)*1e-12/E*365.25; % Gt/yr
        end
       RACMO_clim_err_all(i,j) = rssq(RACMO_clim_range-RACMO_clim_all(i,j));
        
    end
end

% Clear some space
clear lat lon X Y idx time_diff prev_idx sx sy in on smb_layer nan_val cx cy d Yr M E i j smb fn


%% Calculate ensemble mean

% NaN-pad MAR and HIRHAM arrays so they are same length as RACMO
MAR_all = [zeros([numel(GL),12]).*NaN , MAR_all];
HIRHAM_all = [HIRHAM_all , zeros([numel(GL),36]).*NaN];
MAR_clim_all = [zeros([numel(GL),12]).*NaN , MAR_clim_all];
HIRHAM_clim_all = [HIRHAM_clim_all , zeros([numel(GL),36]).*NaN];
MAR_clim_err_all = [zeros([numel(GL),12]).*NaN , MAR_clim_err_all];
HIRHAM_clim_err_all = [HIRHAM_clim_err_all , zeros([numel(GL),36]).*NaN];

% calculate ensemble mean
smb_all = nanmean(cat(3,RACMO_all,MAR_all,HIRHAM_all),3);
smb_clim_all = nanmean(cat(3,RACMO_clim_all,MAR_clim_all,HIRHAM_clim_all),3);
MAR_clim_err_all(isnan(MAR_clim_err_all)) = RACMO_clim_err_all(isnan(MAR_clim_err_all));
HIRHAM_clim_err_all(isnan(HIRHAM_clim_err_all)) = RACMO_clim_err_all(isnan(HIRHAM_clim_err_all));
smb_clim_err_all = sqrt(RACMO_clim_err_all.^2 + MAR_clim_err_all.^2 + HIRHAM_clim_err_all.^2);
time = RACMO_time;

%% Save

% save all
save('ice_shelf_smb_ensemble.mat','MAR_all','HIRHAM_all','RACMO_all',...
    'MAR_clim_all','HIRHAM_clim_all','RACMO_clim_all',...
    'MAR_clim_err_all','HIRHAM_clim_err_all','RACMO_clim_err_all',...
    'smb_all','smb_clim_all','time','shelf_names','A_all_use');

% save for individual ice sheves
for i = 1:numel(GL)
    A_ts = A_all_use(i,:);
    smb_ts = smb_all(i,:);
    RACMO_ts = RACMO_all(i,:);
    MAR_ts = MAR_all(i,:);
    HIRHAM_ts = HIRHAM_all(i,:);
    smb_clim_ts = smb_clim_all(i,:);
    RACMO_clim_ts = RACMO_clim_all(i,:);
    MAR_clim_ts = MAR_clim_all(i,:);
    HIRHAM_clim_ts = HIRHAM_clim_all(i,:);
    smb_clim_err_ts = smb_clim_err_all(i,:);
    RACMO_clim_err_ts = RACMO_clim_err_all(i,:);
    MAR_clim_err_ts = MAR_clim_err_all(i,:);
    HIRHAM_clim_err_ts = HIRHAM_clim_err_all(i,:);
    save(['individual_ice_shelf_SMB_ensemble/' GL(i).Name '_SMB_ensemble.mat'],'A_ts','smb_ts','RACMO_ts','MAR_ts','HIRHAM_ts',...
        'smb_clim_ts','RACMO_clim_ts','MAR_clim_ts','HIRHAM_clim_ts',...
        'smb_clim_err_ts','RACMO_clim_err_ts','MAR_clim_err_ts','HIRHAM_clim_err_ts','time');
end
clear A_ts smb_ts RACMO_ts MAR_ts HIRHAM_ts RACMO_clim_ts MAR_clim_ts HIRHAM_clim_ts smb_clim_ts RACMO_clim_err_ts MAR_clim_err_ts HIRHAM_clim_err_ts smb_clim_err_ts


%% Plot all

if do_plotting == 1
    disp(' ');
    disp('Plotting timeseries...');
    for i = 1:numel(GL)
        disp(['Processing ice shelf ' num2str(i) ': ' GL(i).Name ]);
        % Plot SMB timeseries for this shelf
        figure('units','inches','outerposition',[5 5 8 3.5],'color','w','visible','off')
        plot(time,RACMO_all(i,:),'r'); hold on; datetick('x');
        plot(time,HIRHAM_all(i,:),'b');
        plot(time,MAR_all(i,:),'m');
        plot(time,smb_all(i,:),'k'); 
        legend('RACMO','HIRHAM','MAR','Ensemble','autoupdate','off');
        plot([min(time) max(time)],[0 0],'k','linestyle','--');
        ylabel('SMB (Gt yr^{-1})');
        xtickangle(90);
        yyaxis right
        plot(coast_time_all,A_all(i,:)./(1000*1000),'g');
        ylabel('Area (km^2)');
        ax = gca;
        ax.YAxis(2).Color = 'b';
        export_fig(['individual_ice_shelf_SMB_ensemble/' GL(i).Name '_SMB_timeseries_basic.png'],'-dpng','-r150');
        close

    end
end



